#!/usr/bin/perl
#
# exilist - small exim list manager
# 
# (c) Tom Kistner aka duncanthrax 2002-?
#
# Licensed under the GNU GPL
# see LICENSE for details
#
# ------------------------------------------
# CONFIGURATION:
# Please scroll down 1-2 pages :)
# ------------------------------------------
#

package exilist;
use strict;
use Storable;
use DBI;

BEGIN {
  use Exporter;
  use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

  # set the version for version checking
  $VERSION     = 0.1;
  @ISA         = qw(Exporter);
  @EXPORT      = qw( 
                     $config
                     $dbh
                     &portionize
                     &get_email_addr
                     &restore_session
                     &new_session
                     &store_session
                     &send_message
                     &get_user
                     &get_users
                     &get_list
                     &get_lists
                     &get_list_owner
                     &set_list_owner
                     &get_list_moderators
                     &get_list_subscribers
                     &get_list_pending
                     &subscribe_user
                     &unsubscribe_user
                     &fatal
                     &is_in_array
                     &dv
                     &d
                     &generate_password
                     &update_stats
                     &clean_time
                   );
                   
  %EXPORT_TAGS = ();

  # your exported package globals go here,
  # as well as any optionally exported functions
  @EXPORT_OK   = qw();
  
  use vars qw(
               $config
               $dbh
              );
}


$config = {
# ---------------------------------------------------------------
# CONFIGURATION SECTION
# 
# edit the following to suit your system setup
#

# this is where you have unpacked the distribution

            'home'        => '/var/www/duncanthrax.net/exilist',

# The name of your site
            'sitename'    => 'duncanthrax.net',

# Email address to use as sender for automatic notifications
# This address should be deliverable on your system

            'siteaddr'    => 'exilist@duncanthrax.net',


# The domain part of the web interface http:// address
            
            'href_domain' => 'duncanthrax.net',

# The path part of the web interface http:// address
# (just a slash for an empty path)

            'href_path'   => '/exilist/',

# Logo and Stylesheet locations, relative to 'href_domain'
# and 'href_path'. You can use the provided defaults.
            
            'logo'        => 'html/exilist_logo.gif',
            'stylesheet'  => 'html/exilist_stylesheet.css',

# Absolute path and name of your exim binary
            
            'exim'        => '/usr/sbin/exim',

# Database definition, user and password
            
            'db'          => 'DBI:mysql:database=exilist',
            'db_user'     => 'myusername',
            'db_pass'     => 'mypassword',

# END OF CONFIGURATION
# you can edit the stuff below if you want to change colors
# and usage defaults
# ---------------------------------------------------------------

            
            'defaults'    => {
                                'per_page'              => '10'
                             },
            
            
            'per_page'    => {
                                '5'   => '  5',
                                '10'  => ' 10',
                                '25'  => ' 25',
                                '50'  => ' 50',
                                '100' => '100',
                                '200' => '200'
                             },
            
            'colors'      => {
                                'tooltip_font'          => '#000000',
                                'tooltip_background'    => '#FFFF00',
                                'tooltip_border'        => '#000000',
                                'menu_background_1'     => '#CCCCCC',
                                'menu_background_2'     => '#EEEEEE',
                                'menu_table_background' => '#999999',
                                'menu_font_1'           => '#CC3300',
                                'menu_font_2'           => '#DD5511',
                                'table_background'      => '#999999',
                                'table_header_background' => '#555555',
                                'table_header_foreground' => '#FFFFFF',
                                'table_line_background_1' => '#CCCCCC',
                                'table_line_background_2' => '#DDDDDD',
                                'form_element_background' => '#DDDDDD',
                                'form_element_optional_background'  => '#AAFFAA',
                                'checkbutton_bar_background'  => '#AAAAAA'
                                
                             }
          };


$dbh = DBI->connect($config->{db}, $config->{db_user}, $config->{db_pass});
fatal("Cannot open database ($@).") unless (dv($dbh)); # does not return



sub clean_time {

  my $mytime = shift;
  
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
    localtime($mytime);

  $year+=1900;
  $mon+=1;

  return "$year-$mon-$mday";  

}


# ---------------------------------------------------------------------
# update_stats($tablename, $id, $pairs)

sub update_stats {
  my $table_name = shift;
  my $id = shift;
  my $pairs = shift;
  
  my $sh = $dbh->prepare("SELECT * FROM $table_name WHERE id='$id'");
  $sh->execute;
  my $href = $sh->fetchrow_hashref;
  $sh->finish;
  
  if (dv($href)) {
    
    my $update_req = "UPDATE $table_name SET ";
  
    foreach my $key (keys %{ $pairs } ) {
      
      if ($pairs->{$key} =~ /^set (.+)$/) {
        $href->{$key} = $1;
      }
      else {
        $href->{$key} += $pairs->{$key};
      };
      $update_req .= $key."='".$href->{$key}."', "; 
    };
    
    unless (dv($href->{stats_since})) {
      $href->{stats_since} = time();
      $update_req .= "stats_since='".$href->{stats_since}."', "; 
    };
    
    chop($update_req);chop($update_req);
    $update_req .= " WHERE id='$id'";
  
    $dbh->do($update_req);
  };
  
  return 0;
};



# ---------------------------------------------------------------------
# subscribe_user($list_id,$user_id,$user_status,$force)
# subscribes a user or sets pending status
# returns 0 on failure, 1 on success
sub subscribe_user {

  my $list = get_list(shift);
  my $user = get_user(shift);
  my $force = shift || 0;
  $force = 2 if (dv($force)); # '2' means 'subscribed'
  
  # can't subscribe to closed lists unless forced
  return 0 unless($force || $list->{subscribe_type});

  my $user_status = $user->{lists}->{$list->{id}}->{user_status} || 0;
  my $result = 0;
  my $user_subscription = $force || $list->{subscribe_type};
  
  # list + user exists + non-anon ?
  if (dv($list) && dv($user->{status})) {
    # user is not already subscribed or pending
    #unless (dv($user->{lists}->{$list->{id}}->{user_subscription})) {
      # update existing record
      $result = $dbh->do("UPDATE list_user_link SET user_subscription='$user_subscription' WHERE list_id='$list->{id}' AND user_id='$user->{id}'");
      # try to add new record if this has failed
      # DBI returns '0E0' when no error but no rows are affected !?
      if ($result eq '0E0') {
        $result = $dbh->do("INSERT INTO list_user_link (list_id,user_id,user_status,user_subscription) VALUES($list->{id},$user->{id},$user_status,$user_subscription)");
      };
    #};
  };
  return $user_subscription;
};
  
  
# ---------------------------------------------------------------------
# unsubscribe_user($list_id,$user_id)
# unsubscribes a user or resets pending status
# returns 0 on failure, 1 on success
sub unsubscribe_user {
  my $list = get_list(shift);
  my $user = get_user(shift);  
  
  my $result = 0;
  # list + user exists + non-anon ?
  if (dv($list) && dv($user->{status})) {
    # update existing record
    $result = $dbh->do("UPDATE list_user_link SET user_subscription=0 WHERE list_id=$list->{id} AND user_id=$user->{id}");
  };
  return $result;
};


# ---------------------------------------------------------------------
# get_user($id || $username, [$liststatus])
# retrieves a user record from the database
# returns hash ref
sub get_user {
  print STDERR "get_user(".join(",",@_).")\n";
  my $username = shift;
  my $liststatus = shift;
  $liststatus = 1 unless defined($liststatus);
  my $href = {};
  if ($username =~ /[A-Za-z@]/) {
    my $sh = $dbh->prepare("SELECT * FROM users WHERE address='$username'");
    $sh->execute;
    $href = $sh->fetchrow_hashref;
    $sh->finish;
  }
  else {
    my $sh = $dbh->prepare("SELECT * FROM users WHERE id='$username'");
    $sh->execute;
    $href = $sh->fetchrow_hashref;
    $sh->finish;
  };
  if ($liststatus) {
    # get users status on lists
    if (dv($href)) {
      my $sh = $dbh->prepare("SELECT * FROM list_user_link WHERE user_id=$href->{id}");
      $sh->execute;
      while (my $list_href = $sh->fetchrow_hashref) {
        $href->{lists}->{$list_href->{list_id}} = $list_href;
      };
      $sh->finish;
    };
  };
  return $href;
};


# ---------------------------------------------------------------------
# get_users(list_id)
# returns array of user hash refs
#
# REF->{user_id}
#      {list_id}
#      {user_group}
#      {user_desc}
#      ....
#      {userdata}->{address}
#      {userdata}->{status}
#      {userdata}->{extra}
#      ....

sub get_users {
  my $list_id = shift;
  
  my @entries = ();
  my $sh = $dbh->prepare("SELECT * FROM list_user_link WHERE list_id='$list_id' AND (user_subscription='1' OR user_subscription='2')");
  $sh->execute;
  while (my $row_href = $sh->fetchrow_hashref) {
    push @entries, $row_href;
  };
  $sh->finish;

  foreach (@entries) {
    my $sh = $dbh->prepare("SELECT * FROM users WHERE id='$_->{user_id}'");
    $sh->execute;
    $_->{userdata} = $sh->fetchrow_hashref;
    $sh->finish;
  };
  
  return \@entries;
};

# ---------------------------------------------------------------------
# get_list($list_id)
# returns list href
sub get_list {
  if ((scalar @_) == 1) {
    my $list_id = shift;
    my $sh = $dbh->prepare("SELECT * FROM lists WHERE id=$list_id");
    $sh->execute;
    my $href = $sh->fetchrow_hashref;
    $sh->finish;
    return $href;
  }
  else {
    my $list_name = shift;
    my $list_domain = shift;
    my $sh = $dbh->prepare("SELECT * FROM lists WHERE name='$list_name' AND domain='$list_domain'");
    $sh->execute;
    my $href = $sh->fetchrow_hashref;
    $sh->finish;
    return $href;
  };
};

# ---------------------------------------------------------------------
# get_lists(sortkey,order)
# returns array of list hash refs
sub get_lists {
  my $sortkey = shift;
  my $order = shift;
  if (dv($order)) {
    $order = 'DESC';
  }
  else {
    $order = 'ASC';
  };
  my $sh = $dbh->prepare("SELECT * FROM lists ORDER BY $sortkey $order");
  $sh->execute;
  my @entries = ();
  while (my $row_href = $sh->fetchrow_hashref) {
    push @entries, $row_href;
  };
  $sh->finish;

  return \@entries;
};


# ---------------------------------------------------------------------
# set_list_owner(list_id,user_id)
# returns user struct
sub set_list_owner {
  my $list_id = shift;
  my $user_id = shift;
  
  # get current owner and remove him
  my $cur_owner = get_list_owner($list_id);
  if (dv($cur_owner)) {
    $dbh->do("UPDATE list_user_link SET user_status='0' WHERE list_id='$list_id' AND user_id='$cur_owner->{id}'");
  };
  # update existing record
  my $result = $dbh->do("UPDATE list_user_link SET user_status='2' WHERE list_id='$list_id' AND user_id='$user_id'");
  # try to add new record if this has failed
  # DBI returns '0E0' when no error but no rows are affected !?
  if ($result eq '0E0') {
    $result = $dbh->do("INSERT INTO list_user_link (list_id,user_id,user_status) VALUES($list_id,$user_id,2)");
  };
  return 1;
};


# ---------------------------------------------------------------------
# get_list_owner(list_id)
# returns user struct
sub get_list_owner {
  my $list_id = shift;
  my $href = {};
  my $sh = $dbh->prepare("SELECT * FROM list_user_link WHERE list_id='$list_id' AND user_status='2'");
  $sh->execute;
  $href = $sh->fetchrow_hashref;
  $sh->finish;
  $href = get_user($href->{user_id});
  return $href;
};

# ---------------------------------------------------------------------
# get_list_moderators(list_id)
# returns hash of user struct hashes, with ids as key
sub get_list_moderators {
  my $list_id = shift;
  my $href = {};
  my $sh = $dbh->prepare("SELECT * FROM list_user_link WHERE list_id='$list_id' AND user_status='1'");
  $sh->execute;
  while (my $row_href = $sh->fetchrow_hashref) {
    $href->{$row_href->{user_id}} = get_user($row_href->{user_id});
  };
  $sh->finish;
  return $href;
};

# ---------------------------------------------------------------------
# get_list_subscribers(list_id)
# returns hash of user struct hashes, with ids as key, and number of subscriptions
sub get_list_subscribers {
  my $list_id = shift;
  my $c = 0;
  my $href = {};
  my $sh = $dbh->prepare("SELECT * FROM list_user_link WHERE list_id='$list_id' AND user_subscription='2'");
  $sh->execute;
  while (my $row_href = $sh->fetchrow_hashref) {
    $c++;
    $href->{$row_href->{user_id}} = get_user($row_href->{user_id});
  };
  $sh->finish;
  return ($href,$c);
};

# ---------------------------------------------------------------------
# get_list_pending(list_id)
# returns hash of user struct hashes, with ids as key and number of pending subscriptions
sub get_list_pending {
  my $list_id = shift;
  my $c = 0;
  my $href = {};
  my $sh = $dbh->prepare("SELECT * FROM list_user_link WHERE list_id='$list_id' AND user_subscription='1'");
  $sh->execute;
  while (my $row_href = $sh->fetchrow_hashref) {
    $c++;
    $href->{$row_href->{user_id}} = get_user($row_href->{user_id});
  };
  $sh->finish;
  return ($href,$c);
};

# ---------------------------------------------------------------------
# fatal($cause)
# bombs out with error on STDERR
sub fatal {
  print STDERR $_."\n";
  exit(255);
};

# ---------------------------------------------------------------------
# new_session()
# Creates a blank session and dumps it to disk.
# Returns the session ID
sub new_session {
  my $session = { 
                  id => (sprintf("%08x", (rand 4294967295))),
                  timestamp => (time()),
                  check_ip => 0,
                  ip => $ENV{REMOTE_ADDR},
                  user => 'anonymous'
                };  
  store $session, "$config->{home}/temp/$session->{id}.session";
  return $session->{id};
};

# ---------------------------------------------------------------------
# store_session($session_href,[user_ref])
# saves session to disk
# returns 0 on failure, session href on success
sub store_session {
  my $session = shift;
  # no session, no service
  return 0 unless (dv($session) && ($session =~ /HASH/));
  store $session, "$config->{home}/temp/$session->{id}.session";
  return $session;
};

# ---------------------------------------------------------------------
# restore_session($session_id)
# loads session from disk and updates its timestamp
# returns 0 on failure, session href on success
sub restore_session {
  my $session_id = shift;
  # no session id, no service
  return 0 unless (dv($session_id));
  # check file
  return 0 unless (-e "$config->{home}/temp/$session_id.session");
  # get file
  my $session = retrieve "$config->{home}/temp/$session_id.session";
  # no such session
  return 0 unless (dv($session));
  # check IP address if session requests this
  if ($session->{check_ip}) {
    unless ($ENV{REMOTE_ADDR} eq $session->{ip}) {
      # IP address changed, session invalid
      unlink $config->{home}."/temp/".$session_id;
      return 0;
    };
  };
  # update time marker
  $session->{timestamp} = time();
  store $session,"$config->{home}/temp/$session_id.session";
  return $session;
};

# ---------------------------------------------------------------------
# get_email_addr($string)
# attemps to get a full qualified email address out of $string
# returns first found address, or undef on failure
#
sub get_email_addr {
  my $str = shift;
  $str =~ m/[^A-Za-z0-9+=._-]*([A-Za-z0-9+=._-]+)\@([A-Za-z0-9+=.-]+)[^A-Za-z0-9+=.-]*/;
  my $local_part = $1;
  my $domain = $2;  
  if ($local_part && $domain) {
    return $1.'@'.$2;
  }
  else {
    return undef;
  };
};

# ---------------------------------------------------------------------
# portionize($aref,$per_slice,$max_slice)
#
sub portionize {

  my $aref = shift;
  my $per_slice = shift || 25;
  my $max_slice = shift || 5;
  my $title_key = shift;
  my @a = @{ $aref };
  my $num = scalar @a;
  my $slices = int($num / $per_slice) + 1;
  if ($slices > $max_slice) {
    $per_slice = int($num / $max_slice) + 1;
    $slices = $max_slice;
  };
  # start at 1 -> human compatible
  my $slicecount = 1;
  my $sliceref = {};
  while (@a) {
    my @newslice = ();
    my $slicetitle = "";
    if ($title_key eq "address") {
      $slicetitle = lc(substr($a[0]->{userdata}->{$title_key},0,6));
    }
    else {
      $slicetitle = lc(substr($a[0]->{$title_key},0,6));
    };
    
    my $left = $per_slice;
    while( ($left) && (@a) ) {
      push @newslice, $a[0];
      splice @a,0,1;
      $left--;
    };
    if ($title_key eq "address") {
      $slicetitle .= " .. ".lc(substr($newslice[$#newslice]->{userdata}->{$title_key},0,6));
    }
    else {
      $slicetitle .= " .. ".lc(substr($newslice[$#newslice]->{$title_key},0,6));
    };
      
    $sliceref->{$slicecount}->{title} = $slicetitle;
    $sliceref->{$slicecount}->{aref} = \@newslice;
    $slicecount++;
  };
  return $sliceref;
};


# -------------------------------------------------------------------------
# sends a message to an address
sub send_message {
  my $rcpt = shift;
  my $subject = shift;
  my $message = shift;
  
  open(EXIM,"| $config->{exim} -bS 2>&1 >/dev/null") or add_error("I/O error","cannot open exim for BSMTP input");
  print EXIM "mail from:$config->{siteaddr}\n";
  print EXIM "rcpt to:$rcpt\n";
  print EXIM "data\n";
  print EXIM "From: $config->{sitename} mailing list manager <$config->{siteaddr}>\n";
  print EXIM "To: $rcpt\n";
  print EXIM "Subject: $subject\n";
  print EXIM "\n";
  print EXIM $message;
  print EXIM "\n.\n";
  print EXIM "quit\n"; 
  close(EXIM);
};

# -----------------------------------------------
# is_in_array( entry, arrayref ) 
# checks if "entry" is a member of "array"
# returns entry if yes, empty string if not
# CASELESS !!!
sub is_in_array {
  my $entry = shift;
  my $array = shift;
  unless ($array =~ /ARRAY/) {
    return '';  
  };
  foreach (@{ $array }) {
    return $entry if (lc($_) eq lc($entry));
  };
  return '';  
};

# --------------------------------------------------------------
# dv($var)
# check if var exists, is defined AND has a true value
# returns $var or 0 on failure
sub dv {
  if (exists($_[0]) && defined($_[0]) && ($_[0])) {
    return $_[0];    
  };
  return 0;  
};

# --------------------------------------------------------------
# d($var)
# check if var exists and is not undef 
# returns 1 or 0 on failure
sub d {
  if (exists($_[0]) && defined($_[0])) {
    return 1;    
  };
  return 0;  
};

sub generate_password {
  my @validchars = split //, '23456789ABCDEFGHIJKLMNPQRSTUVWXYZ';
  my $rpass = "";
  for ( 0..(rand(3)+7) ) {
    substr($rpass,$_,1) = $validchars[rand($#validchars+1)];
  }
  return $rpass;
};

1;
