#!/usr/bin/perl -w
#
# exilist - small exim list manager
# 
# (c) Tom Kistner aka duncanthrax 2002-?
#
# Licensed under the GNU GPL
# see LICENSE for details
#
#
use strict;
use CGI;
use exilist;

#use Data::Dumper;

# ---------------------------------------
my $version = '0.3';
# ---------------------------------------

# ************************* START OF MAIN FUNCTION **************************************
# -------------------------------------------------------------------------
# Set up globals to use accross all functions.
my $q = new CGI;
my $cookie = 0;

# Reload the session. Try from cookie first.
my $session = restore_session($q->cookie('session_id'));
unless ($session) {
  # No cookie. Try from CGI var or make a new session.
  my $session_id = $q->param('session_id') || new_session();
  $session = restore_session($session_id) || restore_session(new_session());
};

my %errors = ();
my $new_page_id = undef;
my $new_page_parameters = {};
# Catch global function calls
if (dv($q->param('action_name')) && ($q->param('action_name') eq 'set_page_id')) {
  # jump to a different page, optionally pass parameters
  $new_page_id = $q->param('action_value');
  while ($new_page_id =~ /\&/) {
    $new_page_id =~ s/\&([^&=]+)\=([^&]+)//;
    $new_page_parameters->{$1} = $2;
  };
};


RESTART:
# Load current user from database.
my $user = get_user($session->{user});

# Call the page handler. Default to the front page.
$session->{page_id} = $new_page_id || $q->param('page_id') || $session->{page_id} || 'front';
my $html = fill_template('mainframe',$new_page_parameters);

# Save session.
store_session($session);

# Catch handovers
if ($html =~ /^HASH/) {
  $new_page_id = $html->{page_id};
  $new_page_parameters = $html->{parameters};
  goto RESTART;
};

# Print the page.
if ($cookie) {
  print $q->header( -cookie=>$cookie );  
}
else {
  print $q->header;
};

print $html;

# ************************* END OF MAIN FUNCTION ****************************************


sub fill_template {
  my $template_name = shift;
  my $self = shift || {};
  my $template = load_template($template_name);
  
  while($template =~ m/\<\?(.+?)\?\>/s) {
    my $code = $1;
    my $expression = eval $code;
    
    # if we get a hash ref back, the underlying page
    # wants to hand over to a different page
    if ($expression =~ /^HASH/) {
      return $expression;
    };
    
    if ($@) {
      print STDERR "eval error in '$template_name'","[$code]: $@ \n";
      add_error("eval error in '$template_name'","[$code]: $@ \n");
      $expression = "[eval error]";
    };
    $template =~ s/\<\?.+?\?\>/$expression/s;
  };
  
  return $template;
};

sub load_template {
  my $template_name = shift;
  my $template = "cannot open template '$config->{home}/tmpl/$template_name.tmpl'";
  if (open(TMPL,"< $config->{home}/tmpl/$template_name.tmpl")) {
    undef $/;
    $template = <TMPL>;
    $/ = "\n";
    close(TMPL);
  }
  else {
    add_error("Cannot open template file","cannot open template '$config->{home}/tmpl/$template_name.tmpl'");
  };
  return $template;
};

sub add_error {
  $errors{$_[0]} = $_[1];
};

# ---------------------------------------------------------------------
# make_form_element($element_href)
#
sub make_form_element {

  my $self = shift;
  my $html = '';
  my $explanation = '';
  my $tooltip_picture = '';
  
  if ($self->{explanation}) {
    $explanation = tooltip($self->{name},$self->{explanation});

    $tooltip_picture = $q->td({ -valign=>"top",
    				                    -style=>"font-size: 20px;",
                                -onmouseover=>"javascript:show_tooltip('$self->{name}')",
                                -onmouseout=>"javascript:hide_tooltip('$self->{name}')" },
                          "&nbsp;".
                          "?"
                       );    
  };

  if ($self->{type} eq "dropdown") {
    # -----------------------------------------------------
    # dropdown type
    my $opts = {
                  -name=>$self->{name},
                  -id=>$self->{name},
                  -override=>1,
                  -values=>[ sort {$a cmp $b} keys(%{ $self->{values} }) ],
                  -labels=>$self->{values},
                  -onFocus=>"javascript:document.exilist_form.action_name.value='".$self->{action}."'",
                  -default=>( $q->param($self->{name}) || $self->{default} || 0 )
               };
    
    foreach my $extra_opt (grep /^\-/, keys(%{ $self })) {
      $opts->{$extra_opt} = $self->{$extra_opt};
    };
    
    $html = $q->table({-cellpadding=>"0",-cellspacing=>"0",-border=>"0"},
              $q->Tr(
                $q->td({ -nowrap=>1,-align=>"right",
                         -style=>$self->{textstyle},
                         -valign=>"middle"},
                         "&nbsp;&nbsp;".$self->{text}."&nbsp;&nbsp;"),
                $q->td({-align=>"left",-valign=>"middle"},
                  $explanation,
                  $q->popup_menu( $opts )
                ),
                $tooltip_picture
              )
            );
    
  }
  elsif ($self->{type} eq "textfield") {
    # -----------------------------------------------------
    # textfield type
    my $opts = {
                  -name=>$self->{name},
                  -id=>$self->{name},
                  -override=>1,
                  #-style=>"overflow: hidden;",
                  -wrap=>"off",
                  -onFocus=>"javascript:document.exilist_form.action_name.value='".$self->{action}."'",
                  -value=>( $q->param($self->{name}) || $self->{default} || '' )
               };
    
    foreach my $extra_opt (grep /^\-/, keys(%{ $self })) {
      $opts->{$extra_opt} = $self->{$extra_opt};    
    };
    
    $html = $q->table({-cellpadding=>"0",-cellspacing=>"0",-border=>"0"},
              $q->Tr(
                $q->td({ -nowrap=>1,-align=>"right",
                         -style=>$self->{textstyle},
                         -valign=>"top"},
                         "&nbsp;&nbsp;".$self->{text}."&nbsp;&nbsp;"),
                $q->td({-align=>"left",-valign=>"middle"},
                  $explanation,
                  $q->textarea( $opts )
                ),
                $tooltip_picture
              )
            );

  }
  elsif ($self->{type} eq "string") {
    # -----------------------------------------------------
    # string type
    my $opts = {
                  -name=>$self->{name},
                  -id=>$self->{name},
                  -override=>1,
                  -onFocus=>"javascript:document.exilist_form.action_name.value='".$self->{action}."'",
                  -value=>( $q->param($self->{name}) || $self->{default} || '' )
               };
    
    foreach my $extra_opt (grep /^\-/, keys(%{ $self })) {
      $opts->{$extra_opt} = $self->{$extra_opt};    
    };
    
    $html = $q->table({-cellpadding=>"0",-cellspacing=>"0",-border=>"0"},
              $q->Tr(
                $q->td({ -nowrap=>1,-align=>"right",
                         -style=>$self->{textstyle},
                         -valign=>"middle"},
                         "&nbsp;&nbsp;".$self->{text}."&nbsp;&nbsp;"),
                $q->td({-align=>"left",-valign=>"middle"},
                  $explanation,
                  $q->textfield( $opts )
                ),
                $tooltip_picture
              )
            );

  }
  elsif ($self->{type} eq "password") {
    # -----------------------------------------------------
    # password type
    my $opts = {
                  -name=>$self->{name},
                  -id=>$self->{name},
                  -override=>1,
                  -onFocus=>"javascript:document.exilist_form.action_name.value='".$self->{action}."'",
                  -value=> ( $q->param($self->{name}) || $self->{default} || '' )
               };
    
    foreach my $extra_opt (grep /^\-/, keys(%{ $self })) {
      $opts->{$extra_opt} = $self->{$extra_opt};    
    };
    
    $html = $q->table({-cellpadding=>"0",-cellspacing=>"0",-border=>"0"},
              $q->Tr(
                $q->td({ -nowrap=>"1",
                         -align=>"right",
                         -style=>$self->{textstyle},
                         -valign=>"middle"
                        },"&nbsp;&nbsp;".$self->{text}."&nbsp;&nbsp;"),
                $q->td({-align=>"left",-valign=>"middle"},
                  $explanation,
                  $q->password_field( $opts )
                ),
                $tooltip_picture
              )
            );
  
  }
  elsif ($self->{type} eq "submit") {
    
    my $are_you_sure = "";
    if (dv($self->{are_you_sure})) {
      $are_you_sure = "if (are_you_sure('$self->{are_you_sure}'))";
    };
     
    my $opts = {
                  -name=>$self->{name},
                  -id=>$self->{name},
                  -override=>1,
                  -onClick=>"javascript: $are_you_sure document.exilist_form.action_name.value='$self->{action}';",
                  -value=> $self->{text}
               };
    
    foreach my $extra_opt (grep /^\-/, keys(%{ $self })) {
      $opts->{$extra_opt} = $self->{$extra_opt};    
    };
    
    $html = $q->submit( $opts );
  }
  else {
    $html = "[no such type]";
  };
  
  return $html;
};

# ---------------------------------------------------------------------
# tooltip($name,$html)
#
sub tooltip {
  my $name = shift;
  my $html = shift;
  return $q->div({-name=>$name."_tip",
                  -id=>$name."_tip",
                  -style=> " font-size: 10pt;
                             color: $config->{colors}->{tooltip_font};
                             background-color: $config->{colors}->{tooltip_background};
                             border-style: solid;
                             border-width: 1px;
                             border-color: $config->{colors}->{tooltip_border};
                             width: 200px;
                             padding-top: 5;
                             overflow: hidden;
                             align: left;
                             text-align: left;
                             padding-bottom: 5;
                             padding-left: 5;
                             padding-right: 5;
                             visibility: hidden;
                             position: absolute;
                             z-index:666; "
                  },
                  $html
         ); 
};
